@@ -1,43 +1,49 @@
 package LWP::UserAgent::iTMS_Client;
 
 require 5.006;
-use base qw 'LWP::UserAgent';
-our $VERSION = '0.03';
+use base 'LWP::UserAgent';
+our $VERSION = '0.04';
 
 use strict;
 use warnings;
 use Carp;
+use File::Path;
 use Time::HiRes;
 use XML::Twig;
 use URI::Escape;
-use Digest::MD5 qw 'md5_hex';
+use Digest::MD5 'md5_hex';
 use Compress::Zlib;
 use Crypt::CBC;
 use Crypt::Rijndael;
 use Crypt::AppleTwoFish;
-use MIME::Base64 qw 'encode_base64 decode_base64';
+use MIME::Base64 qw( encode_base64 decode_base64 );
+use Audio::M4P::QuickTime;
 
 # account type determines ID and password (apple vs AOL)
 my %account_type_code = ( apple => 0, aol => 1 );
 
 # country code is number of the store we buy from
 my %country_code = (
-#    Australia       => ???  FIXME
-    Austria         => '143445-2',
-    Belgium         => '143446-2',
-    Canada          => '143455-6',
-    Finland         => '143447',
-    France          => '143442',
-    Germany         => '143443',
-    'Great Britain' => '143444',
-    Greece          => '143448',
-    Ireland         => '143449',
-    Italy           => '143450',
-    Luxembourg      => '143451-2',
-    Netherlands     => '143452',
-    Portugal        => '143453',
-    Spain           => '143454',
-    'United States' => '143441',
+    Australia       => 143460,
+    Austria         => 143445,
+    Belgium         => 143446,
+    Canada          => 143455,
+    Denmark         => 143458,
+    Finland         => 143447,
+    France          => 143442,
+    Germany         => 143443,
+    Greece          => 143448,
+    Ireland         => 143449,
+    Italy           => 143450,
+    Luxembourg      => 143451,
+    Netherlands     => 143452,
+    Norway          => 143457,
+    Portugal        => 143453,
+    Spain           => 143454,
+    Sweden          => 143456,
+    Switzerland     => 143459,
+    UK              => 143444,
+    USA             => 143441,
 );
 
 # for searches
@@ -45,8 +51,8 @@ my %search_topics = (   album => 'albumTerm', artist => 'artistTerm',
                         composer => 'composerTerm', song => 'songTerm', 
                         genre => 'genreIndex', all => 'term' );
                     
-my $all_search_url = 'http://phobos.apple.com/WebObjects/MZSearch.woa/wa/com.apple.jingle.search.DirectAction/search?';
-my $advanced_search_url = 'http://phobos.apple.com/WebObjects/MZSearch.woa/wa/advancedSearchResults?';
+my $all_search_URL = 'http://phobos.apple.com/WebObjects/MZSearch.woa/wa/com.apple.jingle.search.DirectAction/search?';
+my $advanced_search_URL = 'http://phobos.apple.com/WebObjects/MZSearch.woa/wa/advancedSearchResults?';
 
 my %iTMS_genres =   ( "All Genres" => 1, Alternative => 1, Audiobooks => 1,
   Blues => 1, "Children&#39;s Music" => 1, Classical => 1, Comedy => 1, 
@@ -57,9 +63,6 @@ my %iTMS_genres =   ( "All Genres" => 1, Alternative => 1, Audiobooks => 1,
   Soundtrack => 1, 'Spoken Word' => 1, Vocal => 1, World => 1, 
 );
 
-#  for buying
-my $buy_url = 'https://phobos.apple.com/WebObjects/MZFinance.woa/wa/buyProduct?';
-
 # error handling--croak by default
 my $_error = sub { croak shift };
 
@@ -95,7 +98,6 @@ sub request {
     my $hdr = $self->make_request_header($url);
     my $request = new HTTP::Request('GET' => $url . $params, $hdr);
     my $response = $self->SUPER::request($request);
-    my $content;
     if ($response->is_success) {
         # process and decrypt or decompress the response.
         $self->{protocol}->{content} = $response->content;
@@ -132,34 +134,48 @@ sub request {
             $response->content(Compress::Zlib::memGunzip($response->content));
         }
     }
-    else { $self->err($response->status_line . "\n") }
+    else { $self->err($response->status_line) }
     return $response;
 }
 
 #************ public methods unique to LWP::UserAgent::iTMS_Client ***********#
 
+# search the iTMS. Can be done without logging in first
 sub search {
     my($self, %search_terms) = @_;
-    my $search_url = $advanced_search_url;
+    # we could use the urlBag URL here, but hard coded we skip one request.
+    my $search_url = $advanced_search_URL;
     my $params = '';
     my $loops = 0;
     my $had_song = 0;
+    my %not_together = ( artist => 1, composer => 1 );
+    my @together = ();
     while( my($type, $term) = each %search_terms ) {
         if($type eq 'all') {
-            $search_url = $all_search_url;
+            $search_url = $all_search_URL;
             $params = "term=$term";
             last;
         }
+        push @together, $type if $not_together{$type};
+        # kludge around what appears to be an advancedSearch iTMS CGI bug
+        # when artist and composer are both specified in a search
+        if ( (scalar @together) == 2 ) { 
+            return $self->split_search(@together, \%search_terms); 
+        }
         $params .=  '&' if $loops;
         $had_song = 1 if $type eq 'song';
         $params .= $search_topics{$type} . '=' . uri_escape($term);
         $loops++;
     }
-    $params .= '&songTerm=&sp;' unless $had_song; # kludge for iTMS server
+    # kludge for encephalopathic iTMS advancedSearch CGI when no song specified
+    # Music Store, you _need_ to be google spidered...are you dumb on purpose?
+    $params .= '&songTerm=&sp;' unless $had_song; 
     my $results = $self->request($search_url, $params);
     return $self->parse_dict($results->content, 'array/dict');
 }
 
+# get user's iTMS keys--useful for Linux music players like 
+# MPlayer or vlc that need user keys to play m4p's
 sub retrieve_keys_from_iTMS {
     my ($self) = @_;
     my $gu_id;
@@ -177,6 +193,7 @@ sub retrieve_keys_from_iTMS {
     $self->save_keys;
 }
 
+# get keys with temp virtual machine--see jHymn for java version of this
 sub retrieve_keys_with_temp_id {
     my ($self, $callback) = @_;
     $self->gu_id(0);
@@ -191,6 +208,7 @@ sub retrieve_keys_with_temp_id {
     $self->deauthorize_gu_id($self->{protocol}->{gu_id});
 }
 
+# deauthorize virtual machine (so not to waste one of our 5 possible machines)
 sub deauthorize_gu_id {
     my($self, $gu_id) = @_;
     my $user_id = $self->{protocol}->{user_id};
@@ -201,19 +219,119 @@ sub deauthorize_gu_id {
     $self->deauthorize;
 }
 
+# This one is in ALPHA
+# buy music from iTMS--using an already signed up account 
 sub purchase {
-    my($self, $song_id) = @_;
-    return;   # FIXME
-#    my $purchase_url = $buy_url . buyparams . '&creditBalance=' . 
-#      $self->{protocol}->creditBalance . '&creditDisplay=' . 
-#      urllib.quote(self.dspbalance) . '&freeSongBalance=' . 
-#      $self->{protocol}->{fsbalance} .
-#      '&guid=' . $self->gu_id . 
-#      '&rebuy=false&buyWithoutAuthorization=true&wasWarnedAboutFirstTimeBuy=true';
-      
+    my($self, $entry) = @_;
+    # purchase a song (need the 'buyParams' returned from search)
+    my $buy_url = $self->{protocol}->{secure_bag}->{buyProduct} 
+      or $self->err("URL for 'buyProduct' not found in secure url bag.");
+    my $buy_params = $entry->{buyParams}; 
+    my $buy_request_params = '?' . $buy_params . 
+      '&creditBalance=' . $self->{protocol}->{credit_balance} . 
+      '&creditDisplay=' . uri_escape($self->{protocol}->{credit_display}) . 
+      '&freeSongBalance=' . $self->{protocol}->{free_song_balance} . 
+      '&guid=' . $self->gu_id . 
+      '&rebuy=false&buyWithoutAuthorization=true&wasWarnedAboutFirstTimeBuy=true';
+    my $response = $self->request($buy_url, $buy_request_params);
+    my $dict;
+    if($response->is_success) {
+        $dict = $self->parse_xml_response($response->content);
+        my $result_type = $dict->{jingleDocType};
+        $self->err( "Failed to purchase song $entry->{songId}, 
+          $entry->{songName}: " . $dict->{explanation} ) 
+          unless $result_type and $result_type =~ /Success/i;
+        # downloadable is a hash of hashes keyed by download key
+        my $download_url = $dict->{URL} . '?downloadKey=' . $dict->{downloadKey};
+        $self->{protocol}->{downloadable}->{$download_url} = $dict;
+    }
+    else { 
+        $self->err("Failed in purchase request $buy_url$buy_request_params");
+    }
+    $self->download_songs;
+    return $self->{protocol}->{downloadable};
 }
 
-#******************  internal class methods ****************************#
+# This one is in ALPHA
+# download purchased music not yet gotten from iTMS
+# need a list of music before calling this, via purchase or 
+# get_pending_downloads routines
+sub download_songs {
+    my($self) = @_;
+    foreach my $download_url ( keys %{$self->{protocol}->{downloadable}} ) {
+        my $info = $self->{protocol}->{downloadable}->{$download_url};
+        my $key = $info->{encryptionKey};
+        $key = hexToUint($key) if length($key) == 32;
+        next unless length($key) == 16;
+        my $iviv = decode_base64("JOb1Q/OHEFarNPJ/Zf8Adg==");
+        my $response = $self->request($download_url);
+        next unless $response->is_success;       
+        my $alg = new Crypt::Rijndael($key, Crypt::Rijndael::MODE_CBC);
+        $alg->set_iviv($iviv);
+        my $decoded = $alg->decrypt( $response->content, 0, 
+          int(length($response->content) / 16) * 16 );
+        my $sep = $self->{protocol}->{path_sep};
+        my $path = $self->{protocol}->{download_dir} . $sep . 
+          $info->{playlistArtistName} . $sep . $info->{playlistName};
+        my $fname = $info->{songName} . '.m4a';
+        my $new_fh = $self->open_new_pathname($path, $fname);
+        if($new_fh) { 
+            binmode $new_fh; 
+            print $new_fh $decoded;
+            close $new_fh;
+            my $pathname = $path . $sep . $fname;
+            my $qt = new M4P::Quicktime(file => $pathname);
+            $qt->iTMS_MetaInfo($info);
+            $qt->WriteFile($pathname);
+        }
+        else { 
+            $self->err(
+              "Cannot open pathname for song $info->{songName} at $path: $!");
+        }
+    }
+}
+
+# This one is in ALPHA
+# get a list of songs we have purchased but not signed off on downloading yet
+sub get_pending_downloads {
+    my($self) = @_;
+    $self->login unless $self->{protocol}->{secure_bag};
+    my $pending_song_url = $self->{protocol}->{secure_bag}->{pendingSongs} 
+      or $self->err("URL for 'pendingSongs' not found in secure url bag.");
+    my $response = $self->request($pending_song_url, '?guid=' . $self->gu_id);
+    if($response->is_success) {
+        my $dicts = $self->parse_dict($response->content, 'array/dict');
+        foreach my $dict (@{$dicts}) {
+            my $key = $dict->{downloadKey};
+            my $url = $dict->{URL};
+            next unless $key and $url;
+            my $download_url = $url . '?downloadKey=' . $key;
+            $self->{protocol}->{downloadable}->{$download_url} = $dict;
+        }
+    }
+    return $self->download_songs;
+}
+
+# This one is in pre-ALPHA
+# Get a song preview from preview URL returned with a search
+sub preview {
+    my($self, $preview_song) = @_;
+    # download a song preview (need the 'previewURL' returned from a search)
+    my $preview_url = $preview_song->{previewURL};
+    my $preview_name = reverse ( (split /\//, reverse $preview_url)[0] );
+    my $preview = $self->request($preview_url);
+    if($preview->is_success) {
+        my $sep = $self->{protocol}->{path_sep};
+        my $path = $self->{protocol}->{download_dir} . $sep . 'previews';
+        my $new_fh = 
+          $self->open_new_pathname( $path, $preview_name ) or next;
+        binmode $new_fh;
+        print $new_fh $preview->content;
+        close $new_fh;
+    }
+}
+
+#****** internal class methods--interfaces below may change with updates *****#
 
 sub make_request_header {
     my($self, $url) = @_;
@@ -264,7 +382,7 @@ sub make_gu_id {
 sub store_front { 
     my($self, $country) = @_;
     $self->{protocol}->{country_code} = $country_code{$country} if $country;
-    return $self->{protocol}->{country_code} || $country_code{"United States"};
+    return $self->{protocol}->{country_code} || $country_code{USA};
 }
 
 sub drms_dir {
@@ -274,12 +392,20 @@ sub drms_dir {
       $self->{protocol}->{path_sep} );
 }
 
+sub open_new_pathname {
+    my($self, $path, $filename) = @_;
+    mkpath($path);
+    open( my $fh, '>', $path . $self->{protocol}->{path_sep} . $filename )
+      or return;
+    return $fh;
+}
+
 sub save_gu_id {
     # put guID for the auth in a safe place so can de-auth later
     my($self, $guid) = @_;
     return unless $guid and index($guid, '-') < 0;
     open(my $outfh, '>>', $self->drms_dir . "GUID") 
-      or $self->err("Cannot save GUID, WRITE DOWN (base64) $guid:  $!");
+      or $self->err("Cannot save GUID, WRITE DOWN NOW:  $guid   : $!");
     binmode $outfh;
     print $outfh $guid;
     close $outfh;
@@ -290,19 +416,16 @@ sub save_keys {
     my($self) = @_;
     my %user_keys = %{$self->{protocol}->{user_keys}};
     return unless %user_keys;
-    my $num_new_keys = 0;
     my $basename = sprintf("%s/%08X", $self->drms_dir, $self->ds_id);
     foreach my $k (sort { $a <=> $b } keys %user_keys) {
         my $v = $user_keys{$k};
         my $pathname = sprintf("%s.%03d", $basename, $k); 
-        $num_new_keys++ unless -e $pathname;
         open(my $kfh, '>', $pathname) 
           or $self->err("Cannot open $pathname for writing: $!");
         binmode $kfh;
         print $kfh $v;
         close $kfh;
     }
-    $self->{protocol}->{new_key_count} = $num_new_keys;
 }
 
 sub get_saved_keys {
@@ -329,14 +452,6 @@ sub get_saved_keys {
     return $self->{protocol}->{user_keys};
 }
 
-sub get_key_count {
-    my $n = shift->{protocol}->{user_keys};
-    return length %{$n} if $n;
-    return 0;
-}
-
-sub get_new_key_count { return shift->{protocol}->{new_key_count} }
-
 sub compute_validator {
     my($self, $url, $user_agent) = @_;
     my $random = sprintf( "%04X%04X", rand(0x10000), rand(0x10000) );
@@ -346,6 +461,17 @@ sub compute_validator {
     return $random . '-' . uc $digest;
 }
 
+sub split_search {
+    my($self, $term1, $term2, $search_terms) = @_;
+    my %h1 = %{$search_terms};
+    my %h2 = %{$search_terms};
+    delete $h1{$term1};
+    delete $h2{$term2};
+    my $s1 = $self->search(%h1);
+    my $s2 = $self->search(%h2);
+    return song_keyed_intersection($s1, $s2);
+}
+
 sub parse_dict {
     my($self, $content, $path) = @_;
     my @entries;
@@ -356,35 +482,26 @@ sub parse_dict {
         while( $elt = $elt->next_elt('key') ) {
             my $key = $elt->text;
             my $next = $elt->next_elt;
-            last if $next->name =~ /dict/;
             my $value = ($next) ? $next->next_elt_text : 1;
             $entries[$entry_index]->{$key} = $value;
         }
     };
     my $twig = new XML::Twig( TwigHandlers => { $path => $parser } );
     $twig->parse($content);
-    # return reference to an array of hashrefs.
-    # each hashref is a found item in a dict
+    # return reference to array of hashrefs, each a <dict> type of entry
     return \@entries;
 }
 
 sub parse_xml_response {
-    my($self, $xml_response_text) = @_;
-    my %url_bag_read;    
-    my $parser = sub {
-        my($twig, $elm) = @_;
-        my($key, $string);
-        while( $elm = $elm->next_elt('key') ) {
-            next if $elm->text =~ /urlBag/i;
-            $string = $elm->next_elt('string');
-            next unless $string;
-            $url_bag_read{$elm->text} = $string->text;
-            $elm = $string;
-        } 
-    };
-    my $twig = new XML::Twig( Twig_Handlers => { 'plist/dict' => $parser } );
-    $twig->parse($xml_response_text);
-    return \%url_bag_read;
+    # get the data in all XML dicts, combine them, return as hash reference.
+    # use the basic parse_dict() method not this if duplicate keys in dicts
+    my($self, $xml_response) = @_;
+    my $dicts = $self->parse_dict($xml_response, 'dict');
+    my %dict_hash;
+    foreach my $d (@{$dicts}) { 
+        while(my ($k, $v) = each %{$d}) { $dict_hash{$k} = $v }
+    }
+    return \%dict_hash;
 }
 
 sub login {
@@ -398,11 +515,12 @@ sub login {
     $self->err("No guID for login") unless $gu_id;
     my $resp = $self->request("http://phobos.apple.com/storeBag.xml", '')
       or $self->err("Cannot reach iTMS key server phobos.apple.com via network");   
+    $self->{protocol}->{store_bag} = $resp->content;
     $resp = $self->request("http://phobos.apple.com/secureBag.xml", '')
       or $self->err("Cannot retrieve secure bag from iTMS over network");
-    $self->{protocol}->{url_bag} = $self->parse_xml_response($resp->content);
-    my $authAccount = $self->{protocol}->{url_bag}->{authenticateAccount}
-      or $self->err("URL for 'authenticateAccount' not found in iTMS bag.");
+    $self->{protocol}->{secure_bag} = $self->parse_xml_response($resp->content);
+    my $authAccount = $self->{protocol}->{secure_bag}->{authenticateAccount}
+      or $self->err("URL for 'authenticateAccount' not found in secure bag.");
     $self->{protocol}->{login_id} = $user_id;
     my $cgi_params = '?appleId=' . uri_escape($user_id) . '&password=' .
       uri_escape($password) . '&accountKind=' . $account_type . 
@@ -410,14 +528,16 @@ sub login {
     $resp = $self->request($authAccount, $cgi_params)
       or $self->err("Cannot authenticate user $user_id");
     my $auth_response = $self->parse_xml_response($resp->content);
-    my $jingleDocType = $auth_response->{jingleDocType};
     my $customer_message = $auth_response->{customerMessage};
-    if("authenticationsuccess" ne lc($jingleDocType)) 
-      { $self->err("Login failure! Message: $customer_message") }
+    my $jingleDocType = $auth_response->{jingleDocType};
+    $self->err("Login failure! Message: $customer_message")
+      unless $jingleDocType and $jingleDocType =~ /Success$/i;
     $self->{protocol}->{password_token} = $auth_response->{passwordToken};
     $self->{protocol}->{ds_id} = $auth_response->{dsPersonId};
-    $self->err("Bad dsID from login: $self->{protocol}->{ds_id}") 
-      if $self->{protocol}->{ds_id} < 0;
+    $self->{protocol}->{credit_balance} = $auth_response->{creditBalance};
+    $self->{protocol}->{credit_display} = $auth_response->{creditDisplay};
+    $self->{protocol}->{free_song_balance} = $auth_response->{freeSongBalance};
+    $self->{protocol}->{authentication} = $auth_response; 
     return $auth_response;
 }
 
@@ -425,12 +545,12 @@ sub authorize {
     my($self) = @_;
     my $keys = $self->get_saved_keys;
     return $keys if $keys and scalar keys %{$keys} > 0;
-    my $authorizeMachine = $self->{protocol}->{url_bag}->{authorizeMachine}
-      or $self->err("No URL for authorizeMachine found in bag.");
+    my $authorizeMachine = $self->{protocol}->{secure_bag}->{authorizeMachine}
+      or $self->err("No URL for authorizeMachine found in secure bag.");
     my $cgi_params = "?guid=" . $self->{protocol}->{gu_id};    
     my $resp = $self->request($authorizeMachine, $cgi_params)
       or $self->err("Failed to properly access authorizing server over network");
-    my $dict = $self->parse_dict($resp->content, 'plist/dict')->[0];
+    my $dict = $self->parse_xml_response($resp->content);
     my $jingleDocType = $dict->{jingleDocType};
     $self->err("Authorization failure for guID " . $self->{protocol}->{gu_id})
       unless $jingleDocType and $jingleDocType =~ /success$/i;
@@ -444,6 +564,7 @@ sub authorize {
         }
         $self->{protocol}->{user_keys}->{$k} = $bkey;
     }
+    $self->{protocol}->{authentication} = $dict;
     # user_keys references a hash, base index 1, of binary keys
     # with hash keys 1 .. number of keys
     return $self->{protocol}->{user_keys};
@@ -452,22 +573,15 @@ sub authorize {
 sub deauthorize {
     my($self) = @_;
     $self->login;
-    my $deauth_url = $self->{protocol}->{url_bag}->{deauthorizeMachine} 
+    my $deauth_url = $self->{protocol}->{secure_bag}->{deauthorizeMachine} 
       or $self->err("URL for 'deauthorizeMachine' not found in url bag.");
     my $cgi_params = "?guid=" . uri_escape($self->{protocol}->{gu_id});
     my $resp = $self->request($deauth_url, $cgi_params) 
       or $self->err("Could not access $deauth_url over network");
     my $auth_response = $self->parse_xml_response($resp->content);
-    my $jingleDocType = $auth_response->{jingleDocType};
-    my $customerMessage = $auth_response->{customerMessage};
-    unless( lc($jingleDocType) eq 'success' ) 
-      { $self->err("Error: Failed to deauthorize user $self->{gu_id}") }
-}
-
-sub download_songs {
-    my($self) = @_;
-    # download all pending downloadable music.
-    
+    my $msg = $auth_response->{customerMessage};
+    $self->err("Failed to deauthorize user " . $self->{gu_id} . ": $msg")
+      unless $auth_response->{jingleDocType} =~ /Success/i;
 }
 
 sub err { 
@@ -492,6 +606,22 @@ sub hexToUchar {
     return pack 'C*', map { hex } $hex_string =~ m/../g;
 }
 
+sub song_keyed_intersection {
+    my($aref_1, $aref_2) = @_;
+    return unless $aref_1 and $aref_2 and scalar $aref_1 and scalar $aref_2;
+    my @intersection = ();
+    my(%h1, $h, $k);
+    foreach $h (@$aref_1) { 
+        $k = $h->{songId}; 
+        $h1{$k} = $h if $k;
+    }
+    foreach $h (@$aref_2) { 
+        $k = $h->{songId}; 
+        push @intersection, $h1{$k} if $k and $h1{$k};
+    }
+    return \@intersection;
+}
+
 sub progress {
     my($duration, $callback) = @_;
     my $increment = 5;
@@ -500,7 +630,7 @@ sub progress {
         my $state = shift;
         my $char = '=';
         if   ($state =~ /begin/i) { print  "\n", 'Progress: |   ' }
-        elsif($state =~ /end/i)   { print '| :Done!', "\n" }
+        elsif($state =~ /end/i)   { print "\x08\x08\x08", '| :Done!', "\n" }
         else { printf( "\x08\x08\x08%s%02d%%", $char, $state ) }
     };
     $callback ||= $bar;
@@ -523,11 +653,10 @@ LWP::UserAgent::iTMS_Client - libwww-perl client for Apple iTunes music store
     use LWP::UserAgent::iTMS_Client;
     
     # search the Store   
-    my $ua = LWP::UserAgent::iTMS_Client->new(
-      user_id => 'me@you', password => 'pwd');
+    my $ua = LWP::UserAgent::iTMS_Client->new;
     
     my $listings = $ua->search( song => 'apples' );
-    foreach my $album (@{$listings}) { print $album->{songName} }
+    foreach my $song (@{$listings}) { print $song->{songName} }
 
     $listings = $ua->search(artist => 'Vangelis', song => 'long', 
       genre => 'Electronic');
@@ -565,8 +694,7 @@ listings and obtaining the user's keys.
 =item B<new>
 
     # set up new instance for anon search
-    my $ua = 
-      LWP::UserAgent::iTMS_Client->new(user_id => 'me@you', password => 'pwd');
+    my $ua = LWP::UserAgent::iTMS_Client->new;
 
     # set up for login
     my $ua = new LWP::UserAgent::iTMS_Client(
@@ -576,8 +704,8 @@ listings and obtaining the user's keys.
         gu_id => 'CF1121F1.13F11411.11B1F151.1611F111.1DF11711.11811F19.1011F1A1',
     );
 
-Create a new LWP::UserAgent::iTMS_Client object.
-Options are:
+    Create a new LWP::UserAgent::iTMS_Client object.
+    Options are:
 
     account_type
         Either 'apple' or 'aol', this determines where the authentication 
@@ -597,9 +725,14 @@ Options are:
     password (required)
         The user's own (user typed, in iTunes) password.
         
+    error_handler
+        Default error handling is to croak(). This allows alternate behavior by
+        passing the name of a routine which takes a single scalar argument, 
+        the error message.
+
     deauth_wait_secs
-      Reserved, not currently implemented. 
-      This is to be a mandatory wait before deauthorizing a machine.
+        This is a mandatory wait before deauthorizing a machine after
+        a virtual machine is used to get user keys.
       
     country 
         The country where the user's account is based. Determines purchase 
@@ -615,75 +748,101 @@ Options are:
     
     path_sep
         Generally '/'. Path separator for the local OS.
+        
+    download_dir
+        The default location for downloaded music files.
     
 =item B<request>
 
-Sends a request to the iTunes Music Store. Handles encryption and compression, then 
-returns an HTTP::Response object, as an overloaded method of the base LWP::UserAgent. 
-Generally not called directly unless you know what you are doing. 
+    $ua->request('http://phobos.apple.com/WebObjects/MZSearch.woa/wa/com.apple.jingle.search.DirectAction/search', 
+      '?term=beautiful balloon');
+
+    Sends a request to the iTunes Music Store. The first argument is the URL, 
+    the second is the parameter string.Handles encryption and compression, 
+    then returns an HTTP::Response object, as an overloaded method of the base 
+    LWP::UserAgent. Generally not called directly unless you know what you 
+    are doing. 
 
 =item B<search>
 
-use LWP::UserAgent::iTMS_Client;
+    my $results = $ua->search(song => 'Day', composer => 'Lennon');
+    print "Results for song => Day, composer => Lennon:\n";
+    foreach my $a (@{$results1}) { 
+        foreach (sort keys %$a) { print "$_ => ", $a->{$_}, "\n" } 
+    }
 
-my $ua = LWP::UserAgent::iTMS_Client->new;
+    my $results2 = $ua->search(artist => 'Vangelis', song => 'long ago');
+    print "\nResults for artist => Vangelis, song => long ago:\n";
+    foreach my $a (@{$results2}) { 
+        foreach (sort keys %$a) { print "$_ => ", $a->{$_}, "\n" } 
+    }
 
-my $results1 = $ua->search(song => 'Day', composer => 'Lennon');
-print "\nResults for song => regret, artist => New Order:\n";
-foreach my $a (@{$results1}) { 
-    foreach (sort keys %$a) { print "$_ => ", $a->{$_}, "\n" } 
-}
-
-my $results2 = $ua->search(artist => 'Vangelis', song => 'long ago');
-print "\nResults for song => apples:\n";
-foreach my $a (@{$results2}) { 
-   foreach (sort keys %$a) { print "$_ => ", $a->{$_}, "\n" } 
-}
-
-The following types of searches should be supported: album, artist, composer, 
-song, genre, all. If used, 'all' should override other specifications.
+    The following types of searches should be supported: album, artist, 
+    composer, song, genre, all. If used, 'all' should override other 
+    specifications.
 
 =item B<retrieve_keys_from_iTMS>
 
-Get the keys from the Store. Attempts to be compatible with key locations used 
-by default by the Videolan project's media player (FairKeys compatibility). 
-This should generally be used with a gu_id known by the user, preferentially 
-one given as a gu_id => 11111111.11111111.11111111.11111111.11111111.11111111
-(6 8-digit hex numbers separated by periods) argument to new.
+    $ua->retrieve_keys_from_iTMS;
+
+    Get the keys from the Store. Attempts to be compatible with key locations 
+    used by default by the Videolan project's media player (FairKeys 
+    compatibility). This should generally be used with a gu_id known by the 
+    user, preferentially one given as a 
+    gu_id => 11111111.11111111.11111111.11111111.11111111.11111111
+    (6 8-digit hex numbers separated by periods) argument to new.
 
 =item B<deauthorize_gu_id>
 
-$ua->deauthorize_gu_id($id);
-
-Deauthorize the machine used to get the keys.
+    $ua->deauthorize_gu_id($id);
 
+    Deauthorize the machine used to get the keys.
 
 =item B<retrieve_keys_with_temp_id>
 
-ua->retrieve_keys_with_temp_id(\&callback);
+    $ua->retrieve_keys_with_temp_id;
+    $ua->retrieve_keys_with_temp_id(\&callback);
   
-Create a temporary machine ID (you need to have one of your 5 machine 
-useages for iTunes available), get the keys with this virtual machine's 
-authorization, then deauthorize. Note that since this may result in an 
-additional key being created, you should limit the number of times you 
-do this. If you generally only purchase music on one or two machines 
-that do not change ID's, and only play copied music on your other 
-(iPod?) machines, once downloading your keys may be enough. The program
-will display a progress bar betwwen key retrieval and deauthorization. The
-optional argument is to allow custom display of the wait period, which by
-default prints to stdout. The optional callback routine must accept a single 
-argument which may have the values 'begin', an integer between 0 and 100, 
-and 'end'.
+    Create a temporary machine ID (you need to have one of your 5 machine 
+    usages for iTunes available), get the keys with this virtual machine's 
+    authorization, then deauthorize. Note that since this may result in an 
+    additional key being created, you should limit the number of times you 
+    do this. If you generally only purchase music on one or two machines 
+    that do not change ID's, and only play copied music on your other 
+    (iPod?) machines, once downloading your keys may be enough. The program
+    will display a progress bar betwwen key retrieval and deauthorization. The
+    optional argument is to allow custom display of the wait period, which by
+    default prints to stdout. The optional callback routine must accept a single 
+    argument which may have the values 'begin', an integer between 0 and 100, 
+    and 'end'.
 
 =item B<purchase>
 
-  $ua->purchase($song_id);
+    $ua->purchase($song_id);  # # This one is in ALPHA
   
-  Not yet working, will be soon we hope
+    Purchase and download a song, by song id number or 'songId' as a search 
+    result (use search to find the song and then get the songId).
+
+=item B<download_songs>
+
+    $ua->download_songs; # # This one is in ALPHA
+
+    Download any songs pending for the user, including those just purchased.
+
+=item B<preview>
+
+    # This one is in ALPHA
+    $ua->preview($song);
+    
+    Download a preview for a song entry, if available. $song is a reference
+    to a hash for a song returned by the search method.
 
 =head1 BUGS
 
-The searches do not work if both 'composer' and 'artist' are specified.
+The searches under 'artist' are currently crippled, due to a server issue. 
+It seems that the artist name is seldom presently part of song metadata. There 
+is a numeric artistId entry, but I don't currently have an index of artists 
+for lookup.
 
 Overuse of the B<purchase> routine might allow you to spend more on music 
 than you intended. This might be a bug, from the perspective of your budget. 
